package com.ruoyi.project.asset.service.impl;

import java.util.List;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.ruoyi.project.asset.mapper.TSpecdevAssetMapper;
import com.ruoyi.project.asset.domain.TSpecdevAsset;
import com.ruoyi.project.asset.service.ITSpecdevAssetService;

/**
 * 资产与运营Service业务层处理
 *
 * @author ruoyi
 * @date 2022-10-12
 */
@Service
public class TSpecdevAssetServiceImpl implements ITSpecdevAssetService
{
    @Autowired
    private TSpecdevAssetMapper tSpecdevAssetMapper;

    /**
     * 查询资产与运营
     *
     * @param id 资产与运营ID
     * @return 资产与运营
     */
    @Override
    public TSpecdevAsset selectTSpecdevAssetById(Long id)
    {
        return tSpecdevAssetMapper.selectTSpecdevAssetById(id);
    }

    /**
     * 查询资产与运营列表
     *
     * @param tSpecdevAsset 资产与运营
     * @return 资产与运营
     */
    @Override
    public List<TSpecdevAsset> selectTSpecdevAssetList(TSpecdevAsset tSpecdevAsset)
    {
        return tSpecdevAssetMapper.selectTSpecdevAssetList(tSpecdevAsset);
    }

    /**
     * 新增资产与运营
     *
     * @param tSpecdevAsset 资产与运营
     * @return 结果
     */
    @Override
    public int insertTSpecdevAsset(TSpecdevAsset tSpecdevAsset)
    {
        return tSpecdevAssetMapper.insertTSpecdevAsset(tSpecdevAsset);
    }

    /**
     * 修改资产与运营
     *
     * @param tSpecdevAsset 资产与运营
     * @return 结果
     */
    @Override
    public int updateTSpecdevAsset(TSpecdevAsset tSpecdevAsset)
    {
        return tSpecdevAssetMapper.updateTSpecdevAsset(tSpecdevAsset);
    }

    /**
     * 批量删除资产与运营
     *
     * @param ids 需要删除的资产与运营ID
     * @return 结果
     */
    @Override
    public int deleteTSpecdevAssetByIds(Long[] ids)
    {
        return tSpecdevAssetMapper.deleteTSpecdevAssetByIds(ids);
    }

    /**
     * 删除资产与运营信息
     *
     * @param id 资产与运营ID
     * @return 结果
     */
    @Override
    public int deleteTSpecdevAssetById(Long id)
    {
        return tSpecdevAssetMapper.deleteTSpecdevAssetById(id);
    }
}
